@@ -12,21 +12,6 @@ r"""
     Oracle version 8 through current (11g at the time of this writing) are
     supported.
 
-Connect Arguments
------------------
-
-The dialect supports several :func:`~sqlalchemy.create_engine()` arguments
-which affect the behavior of the dialect regardless of driver in use.
-
-* ``use_ansi`` - Use ANSI JOIN constructs (see the section on Oracle 8).
-  Defaults to ``True``.  If ``False``, Oracle-8 compatible constructs are used
-  for joins.
-
-* ``optimize_limits`` - defaults to ``False``. see the section on
-  LIMIT/OFFSET.
-
-* ``use_binds_for_limits`` - defaults to ``True``.  see the section on
-  LIMIT/OFFSET.
 
 Auto Increment Behavior
 -----------------------
@@ -53,6 +38,36 @@ This step is also required when using table reflection, i.e. autoload=True::
         autoload=True
   )
 
+Transaction Isolation Level / Autocommit
+----------------------------------------
+
+The Oracle database supports "READ COMMITTED" and "SERIALIZABLE" modes
+of isolation, however the SQLAlchemy Oracle dialect currently only has
+explicit support for "READ COMMITTED".  It is possible to emit a
+"SET TRANSACTION" statement on a connection in order to use SERIALIZABLE
+isolation, however the SQLAlchemy dialect will remain unaware of this setting,
+such as if the :meth:`_engine.Connection.get_isolation_level` method is used;
+this method is hardcoded to return "READ COMMITTED" right now.
+
+The AUTOCOMMIT isolation level is also supported by the cx_Oracle dialect.
+
+To set using per-connection execution options::
+
+    connection = engine.connect()
+    connection = connection.execution_options(
+        isolation_level="AUTOCOMMIT"
+    )
+
+Valid values for ``isolation_level`` include:
+
+* ``READ COMMITTED``
+* ``AUTOCOMMIT``
+
+
+.. versionadded:: 1.3.16 added support for AUTOCOMMIT to the cx_oracle dialect
+   as well as the notion of a default isolation level, currently harcoded
+   to "READ COMMITTED".
+
 Identifier Casing
 -----------------
 
@@ -95,11 +110,13 @@ For the duration of the SQLAlchemy 1.3 series, the default max identifier
 length will remain at 30, even if compatibility version 12.2 or greater is in
 use.  When the newer version is detected, a warning will be emitted upon first
 connect, which refers the user to make use of the
-:paramref:`.create_engine.max_identifier_length` parameter in order to assure
+:paramref:`_sa.create_engine.max_identifier_length`
+parameter in order to assure
 forwards compatibility with SQLAlchemy 1.4, which will be changing this value
 to 128 when compatibility version 12.2 or greater is detected.
 
-Using :paramref:`.create_engine.max_identifier_length`, the effective identifier
+Using :paramref:`_sa.create_engine.max_identifier_length`,
+the effective identifier
 length used by the SQLAlchemy dialect will be used as given, overriding the
 current default value of 30, so that when Oracle 12.2 or greater is used, the
 newer identifier length may be taken advantage of::
@@ -162,14 +179,15 @@ when the identifier length is changed without the name of the index or
 constraint first being adjusted.
 
 Therefore, applications are strongly advised to make use of
-:paramref:`.create_engine.max_identifier_length` in order to maintain control
+:paramref:`_sa.create_engine.max_identifier_length`
+in order to maintain control
 of the generation of truncated names, and to fully review and test all database
 migrations in a staging environment when changing this value to ensure that the
 impact of this change has been mitigated.
 
 
 .. versionadded:: 1.3.9 Added the
-   :paramref:`.create_engine.max_identifier_length` parameter; the Oracle
+   :paramref:`_sa.create_engine.max_identifier_length` parameter; the Oracle
    dialect now detects compatibility version 12.2 or greater and warns
    about upcoming max identitifier length changes in SQLAlchemy 1.4.
 
@@ -186,12 +204,12 @@ There are two options which affect its behavior:
 
 * the "FIRST ROWS()" optimization keyword is not used by default.  To enable
   the usage of this optimization directive, specify ``optimize_limits=True``
-  to :func:`.create_engine`.
+  to :func:`_sa.create_engine`.
 * the values passed for the limit/offset are sent as bound parameters.   Some
   users have observed that Oracle produces a poor query plan when the values
   are sent as binds and not rendered literally.   To render the limit/offset
   values literally within the SQL statement, specify
-  ``use_binds_for_limits=False`` to :func:`.create_engine`.
+  ``use_binds_for_limits=False`` to :func:`_sa.create_engine`.
 
 Some users have reported better performance when the entirely different
 approach of a window query is used, i.e. ROW_NUMBER() OVER (ORDER BY), to
@@ -220,7 +238,7 @@ on the Oracle backend.  By default, "implicit returning" typically only
 fetches the value of a single ``nextval(some_seq)`` expression embedded into
 an INSERT in order to increment a sequence within an INSERT statement and get
 the value back at the same time. To disable this feature across the board,
-specify ``implicit_returning=False`` to :func:`.create_engine`::
+specify ``implicit_returning=False`` to :func:`_sa.create_engine`::
 
     engine = create_engine("oracle://scott:tiger@dsn",
                            implicit_returning=False)
@@ -282,7 +300,7 @@ Synonym/DBLINK Reflection
 When using reflection with Table objects, the dialect can optionally search
 for tables indicated by synonyms, either in local or remote schemas or
 accessed over DBLINK, by passing the flag ``oracle_resolve_synonyms=True`` as
-a keyword argument to the :class:`.Table` construct::
+a keyword argument to the :class:`_schema.Table` construct::
 
     some_table = Table('some_table', autoload=True,
                                 autoload_with=some_engine,
@@ -296,8 +314,8 @@ knows how to locate the table's information using DBLINK syntax(e.g.
 ``@dblink``).
 
 ``oracle_resolve_synonyms`` is accepted wherever reflection arguments are
-accepted, including methods such as :meth:`.MetaData.reflect` and
-:meth:`.Inspector.get_columns`.
+accepted, including methods such as :meth:`_schema.MetaData.reflect` and
+:meth:`_reflection.Inspector.get_columns`.
 
 If synonyms are not in use, this flag should be left disabled.
 
@@ -310,18 +328,22 @@ The Oracle dialect can return information about foreign key, unique, and
 CHECK constraints, as well as indexes on tables.
 
 Raw information regarding these constraints can be acquired using
-:meth:`.Inspector.get_foreign_keys`, :meth:`.Inspector.get_unique_constraints`,
-:meth:`.Inspector.get_check_constraints`, and :meth:`.Inspector.get_indexes`.
+:meth:`_reflection.Inspector.get_foreign_keys`,
+:meth:`_reflection.Inspector.get_unique_constraints`,
+:meth:`_reflection.Inspector.get_check_constraints`, and
+:meth:`_reflection.Inspector.get_indexes`.
 
 .. versionchanged:: 1.2  The Oracle dialect can now reflect UNIQUE and
    CHECK constraints.
 
-When using reflection at the :class:`.Table` level, the :class:`.Table`
+When using reflection at the :class:`_schema.Table` level, the
+:class:`_schema.Table`
 will also include these constraints.
 
 Note the following caveats:
 
-* When using the :meth:`.Inspector.get_check_constraints` method, Oracle
+* When using the :meth:`_reflection.Inspector.get_check_constraints` method,
+  Oracle
   builds a special "IS NOT NULL" constraint for columns that specify
   "NOT NULL".  This constraint is **not** returned by default; to include
   the "IS NOT NULL" constraints, pass the flag ``include_all=True``::
@@ -333,11 +355,13 @@ Note the following caveats:
       all_check_constraints = inspector.get_check_constraints(
           "some_table", include_all=True)
 
-* in most cases, when reflecting a :class:`.Table`, a UNIQUE constraint will
+* in most cases, when reflecting a :class:`_schema.Table`,
+  a UNIQUE constraint will
   **not** be available as a :class:`.UniqueConstraint` object, as Oracle
   mirrors unique constraints with a UNIQUE index in most cases (the exception
   seems to be when two or more unique constraints represent the same columns);
-  the :class:`.Table` will instead represent these using :class:`.Index`
+  the :class:`_schema.Table` will instead represent these using
+  :class:`.Index`
   with the ``unique=True`` flag set.
 
 * Oracle creates an implicit index for the primary key of a table; this index
@@ -349,11 +373,12 @@ Note the following caveats:
 Table names with SYSTEM/SYSAUX tablespaces
 -------------------------------------------
 
-The :meth:`.Inspector.get_table_names` and
-:meth:`.Inspector.get_temp_table_names`
+The :meth:`_reflection.Inspector.get_table_names` and
+:meth:`_reflection.Inspector.get_temp_table_names`
 methods each return a list of table names for the current engine. These methods
 are also part of the reflection which occurs within an operation such as
-:meth:`.MetaData.reflect`.  By default, these operations exclude the ``SYSTEM``
+:meth:`_schema.MetaData.reflect`.  By default,
+these operations exclude the ``SYSTEM``
 and ``SYSAUX`` tablespaces from the operation.   In order to change this, the
 default list of tablespaces excluded can be changed at the engine level using
 the ``exclude_tablespaces`` parameter::
@@ -370,15 +395,15 @@ DateTime Compatibility
 
 Oracle has no datatype known as ``DATETIME``, it instead has only ``DATE``,
 which can actually store a date and time value.  For this reason, the Oracle
-dialect provides a type :class:`.oracle.DATE` which is a subclass of
+dialect provides a type :class:`_oracle.DATE` which is a subclass of
 :class:`.DateTime`.   This type has no special behavior, and is only
 present as a "marker" for this type; additionally, when a database column
 is reflected and the type is reported as ``DATE``, the time-supporting
-:class:`.oracle.DATE` type is used.
+:class:`_oracle.DATE` type is used.
 
-.. versionchanged:: 0.9.4 Added :class:`.oracle.DATE` to subclass
+.. versionchanged:: 0.9.4 Added :class:`_oracle.DATE` to subclass
    :class:`.DateTime`.  This is a change as previous versions
-   would reflect a ``DATE`` column as :class:`.types.DATE`, which subclasses
+   would reflect a ``DATE`` column as :class:`_types.DATE`, which subclasses
    :class:`.Date`.   The only significance here is for schemes that are
    examining the type of column for use in special Python translations or
    for migrating schemas to other database backends.
@@ -389,7 +414,7 @@ Oracle Table Options
 -------------------------
 
 The CREATE TABLE phrase supports the following options with Oracle
-in conjunction with the :class:`.Table` construct:
+in conjunction with the :class:`_schema.Table` construct:
 
 
 * ``ON COMMIT``::
@@ -561,7 +586,7 @@ class DATE(sqltypes.DateTime):
     """Provide the oracle DATE type.
 
     This type has no special Python behavior, except that it subclasses
-    :class:`.types.DateTime`; this is to suit the fact that the Oracle
+    :class:`_types.DateTime`; this is to suit the fact that the Oracle
     ``DATE`` type supports a time value.
 
     .. versionadded:: 0.9.4
@@ -1093,6 +1118,18 @@ class OracleCompiler(compiler.SQLCompiler):
 
         return tmp
 
+    def visit_is_distinct_from_binary(self, binary, operator, **kw):
+        return "DECODE(%s, %s, 0, 1) = 1" % (
+            self.process(binary.left),
+            self.process(binary.right),
+        )
+
+    def visit_isnot_distinct_from_binary(self, binary, operator, **kw):
+        return "DECODE(%s, %s, 0, 1) = 0" % (
+            self.process(binary.left),
+            self.process(binary.right),
+        )
+
 
 class OracleDDLCompiler(compiler.DDLCompiler):
     def define_constraint_cascades(self, constraint):
@@ -1353,6 +1390,20 @@ class OracleDialect(default.DefaultDialect):
             connection, additional_tests
         )
 
+    _isolation_lookup = ["READ COMMITTED"]
+
+    def get_isolation_level(self, connection):
+        return "READ COMMITTED"
+
+    def set_isolation_level(self, connection, level):
+        # prior to adding AUTOCOMMIT support for cx_Oracle, the Oracle dialect
+        # had no notion of setting the isolation level.  As Oracle
+        # does not have a straightforward way of getting the isolation level
+        # if a server-side transaction is not yet in progress, we currently
+        # hardcode to only support "READ COMMITTED" and "AUTOCOMMIT" at the
+        # cx_oracle level.  See #5200.
+        pass
+
     def has_table(self, connection, table_name, schema=None):
         if not schema:
             schema = self.default_schema_name
@@ -1627,13 +1678,14 @@ class OracleDialect(default.DefaultDialect):
         text = """
             SELECT col.column_name, col.data_type, col.%(char_length_col)s,
               col.data_precision, col.data_scale, col.nullable,
-              col.data_default, com.comments\
-            FROM all_tab_columns%(dblink)s col
+              col.data_default, com.comments, col.virtual_column\
+            FROM all_tab_cols%(dblink)s col
             LEFT JOIN all_col_comments%(dblink)s com
             ON col.table_name = com.table_name
             AND col.column_name = com.column_name
             AND col.owner = com.owner
             WHERE col.table_name = :table_name
+            AND col.hidden_column = 'NO'
         """
         if schema is not None:
             params["owner"] = schema
@@ -1653,6 +1705,7 @@ class OracleDialect(default.DefaultDialect):
             nullable = row[5] == "Y"
             default = row[6]
             comment = row[7]
+            generated = row[8]
 
             if coltype == "NUMBER":
                 if precision is None and scale == 0:
@@ -1677,6 +1730,12 @@ class OracleDialect(default.DefaultDialect):
                     )
                     coltype = sqltypes.NULLTYPE
 
+            if generated == "YES":
+                computed = dict(sqltext=default)
+                default = None
+            else:
+                computed = None
+
             cdict = {
                 "name": colname,
                 "type": coltype,
@@ -1687,6 +1746,8 @@ class OracleDialect(default.DefaultDialect):
             }
             if orig_colname.lower() == orig_colname:
                 cdict["quote"] = True
+            if computed is not None:
+                cdict["computed"] = computed
 
             columns.append(cdict)
         return columns
@@ -1712,13 +1773,18 @@ class OracleDialect(default.DefaultDialect):
             info_cache=info_cache,
         )
 
+        if not schema:
+            schema = self.default_schema_name
+
         COMMENT_SQL = """
             SELECT comments
             FROM all_tab_comments
-            WHERE table_name = :table_name
+            WHERE table_name = :table_name AND owner = :schema_name
         """
 
-        c = connection.execute(sql.text(COMMENT_SQL), table_name=table_name)
+        c = connection.execute(
+            sql.text(COMMENT_SQL), table_name=table_name, schema_name=schema
+        )
         return {"text": c.scalar()}
 
     @reflection.cache
