@@ -8,6 +8,7 @@ from sqlalchemy import CHAR
 from sqlalchemy import CheckConstraint
 from sqlalchemy import CLOB
 from sqlalchemy import Column
+from sqlalchemy import Computed
 from sqlalchemy import DATE
 from sqlalchemy import Date
 from sqlalchemy import DATETIME
@@ -37,6 +38,7 @@ from sqlalchemy import SmallInteger
 from sqlalchemy import sql
 from sqlalchemy import String
 from sqlalchemy import Table
+from sqlalchemy import testing
 from sqlalchemy import TEXT
 from sqlalchemy import TIME
 from sqlalchemy import Time
@@ -385,6 +387,28 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
         )
         self.assert_compile(sql.delete(a1), "DELETE FROM t1 AS a1")
 
+    @testing.combinations(
+        ("no_persisted", "", "ignore"),
+        ("persisted_none", "", None),
+        ("persisted_true", " STORED", True),
+        ("persisted_false", " VIRTUAL", False),
+        id_="iaa",
+    )
+    def test_column_computed(self, text, persisted):
+        m = MetaData()
+        kwargs = {"persisted": persisted} if persisted != "ignore" else {}
+        t = Table(
+            "t",
+            m,
+            Column("x", Integer),
+            Column("y", Integer, Computed("x + 2", **kwargs)),
+        )
+        self.assert_compile(
+            schema.CreateTable(t),
+            "CREATE TABLE t (x INTEGER, y INTEGER GENERATED "
+            "ALWAYS AS (x + 2)%s)" % text,
+        )
+
 
 class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
 
@@ -454,32 +478,32 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
             {"param_1": 10},
         )
 
-    def test_varchar_raise(self):
-        for type_ in (
-            String,
-            VARCHAR,
-            String(),
-            VARCHAR(),
-            NVARCHAR(),
-            Unicode,
-            Unicode(),
-        ):
-            type_ = sqltypes.to_instance(type_)
-            assert_raises_message(
-                exc.CompileError,
-                "VARCHAR requires a length on dialect mysql",
-                type_.compile,
-                dialect=mysql.dialect(),
-            )
+    @testing.combinations(
+        (String,),
+        (VARCHAR,),
+        (String(),),
+        (VARCHAR(),),
+        (NVARCHAR(),),
+        (Unicode,),
+        (Unicode(),),
+    )
+    def test_varchar_raise(self, type_):
+        type_ = sqltypes.to_instance(type_)
+        assert_raises_message(
+            exc.CompileError,
+            "VARCHAR requires a length on dialect mysql",
+            type_.compile,
+            dialect=mysql.dialect(),
+        )
 
-            t1 = Table("sometable", MetaData(), Column("somecolumn", type_))
-            assert_raises_message(
-                exc.CompileError,
-                r"\(in table 'sometable', column 'somecolumn'\)\: "
-                r"(?:N)?VARCHAR requires a length on dialect mysql",
-                schema.CreateTable(t1).compile,
-                dialect=mysql.dialect(),
-            )
+        t1 = Table("sometable", MetaData(), Column("somecolumn", type_))
+        assert_raises_message(
+            exc.CompileError,
+            r"\(in table 'sometable', column 'somecolumn'\)\: "
+            r"(?:N)?VARCHAR requires a length on dialect mysql",
+            schema.CreateTable(t1).compile,
+            dialect=mysql.dialect(),
+        )
 
     def test_update_limit(self):
         t = sql.table("t", sql.column("col1"), sql.column("col2"))
@@ -513,75 +537,73 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_sysdate(self):
         self.assert_compile(func.sysdate(), "SYSDATE()")
 
-    def test_cast(self):
+    m = mysql
+
+    @testing.combinations(
+        (Integer, "CAST(t.col AS SIGNED INTEGER)"),
+        (INT, "CAST(t.col AS SIGNED INTEGER)"),
+        (m.MSInteger, "CAST(t.col AS SIGNED INTEGER)"),
+        (m.MSInteger(unsigned=True), "CAST(t.col AS UNSIGNED INTEGER)"),
+        (SmallInteger, "CAST(t.col AS SIGNED INTEGER)"),
+        (m.MSSmallInteger, "CAST(t.col AS SIGNED INTEGER)"),
+        (m.MSTinyInteger, "CAST(t.col AS SIGNED INTEGER)"),
+        # 'SIGNED INTEGER' is a bigint, so this is ok.
+        (m.MSBigInteger, "CAST(t.col AS SIGNED INTEGER)"),
+        (m.MSBigInteger(unsigned=False), "CAST(t.col AS SIGNED INTEGER)"),
+        (m.MSBigInteger(unsigned=True), "CAST(t.col AS UNSIGNED INTEGER)"),
+        # this is kind of sucky.  thank you default arguments!
+        (NUMERIC, "CAST(t.col AS DECIMAL)"),
+        (DECIMAL, "CAST(t.col AS DECIMAL)"),
+        (Numeric, "CAST(t.col AS DECIMAL)"),
+        (m.MSNumeric, "CAST(t.col AS DECIMAL)"),
+        (m.MSDecimal, "CAST(t.col AS DECIMAL)"),
+        (TIMESTAMP, "CAST(t.col AS DATETIME)"),
+        (DATETIME, "CAST(t.col AS DATETIME)"),
+        (DATE, "CAST(t.col AS DATE)"),
+        (TIME, "CAST(t.col AS TIME)"),
+        (DateTime, "CAST(t.col AS DATETIME)"),
+        (Date, "CAST(t.col AS DATE)"),
+        (Time, "CAST(t.col AS TIME)"),
+        (DateTime, "CAST(t.col AS DATETIME)"),
+        (Date, "CAST(t.col AS DATE)"),
+        (m.MSTime, "CAST(t.col AS TIME)"),
+        (m.MSTimeStamp, "CAST(t.col AS DATETIME)"),
+        (String, "CAST(t.col AS CHAR)"),
+        (Unicode, "CAST(t.col AS CHAR)"),
+        (UnicodeText, "CAST(t.col AS CHAR)"),
+        (VARCHAR, "CAST(t.col AS CHAR)"),
+        (NCHAR, "CAST(t.col AS CHAR)"),
+        (CHAR, "CAST(t.col AS CHAR)"),
+        (m.CHAR(charset="utf8"), "CAST(t.col AS CHAR CHARACTER SET utf8)"),
+        (CLOB, "CAST(t.col AS CHAR)"),
+        (TEXT, "CAST(t.col AS CHAR)"),
+        (m.TEXT(charset="utf8"), "CAST(t.col AS CHAR CHARACTER SET utf8)"),
+        (String(32), "CAST(t.col AS CHAR(32))"),
+        (Unicode(32), "CAST(t.col AS CHAR(32))"),
+        (CHAR(32), "CAST(t.col AS CHAR(32))"),
+        (m.MSString, "CAST(t.col AS CHAR)"),
+        (m.MSText, "CAST(t.col AS CHAR)"),
+        (m.MSTinyText, "CAST(t.col AS CHAR)"),
+        (m.MSMediumText, "CAST(t.col AS CHAR)"),
+        (m.MSLongText, "CAST(t.col AS CHAR)"),
+        (m.MSNChar, "CAST(t.col AS CHAR)"),
+        (m.MSNVarChar, "CAST(t.col AS CHAR)"),
+        (LargeBinary, "CAST(t.col AS BINARY)"),
+        (BLOB, "CAST(t.col AS BINARY)"),
+        (m.MSBlob, "CAST(t.col AS BINARY)"),
+        (m.MSBlob(32), "CAST(t.col AS BINARY)"),
+        (m.MSTinyBlob, "CAST(t.col AS BINARY)"),
+        (m.MSMediumBlob, "CAST(t.col AS BINARY)"),
+        (m.MSLongBlob, "CAST(t.col AS BINARY)"),
+        (m.MSBinary, "CAST(t.col AS BINARY)"),
+        (m.MSBinary(32), "CAST(t.col AS BINARY)"),
+        (m.MSVarBinary, "CAST(t.col AS BINARY)"),
+        (m.MSVarBinary(32), "CAST(t.col AS BINARY)"),
+        (Interval, "CAST(t.col AS DATETIME)"),
+    )
+    def test_cast(self, type_, expected):
         t = sql.table("t", sql.column("col"))
-        m = mysql
-
-        specs = [
-            (Integer, "CAST(t.col AS SIGNED INTEGER)"),
-            (INT, "CAST(t.col AS SIGNED INTEGER)"),
-            (m.MSInteger, "CAST(t.col AS SIGNED INTEGER)"),
-            (m.MSInteger(unsigned=True), "CAST(t.col AS UNSIGNED INTEGER)"),
-            (SmallInteger, "CAST(t.col AS SIGNED INTEGER)"),
-            (m.MSSmallInteger, "CAST(t.col AS SIGNED INTEGER)"),
-            (m.MSTinyInteger, "CAST(t.col AS SIGNED INTEGER)"),
-            # 'SIGNED INTEGER' is a bigint, so this is ok.
-            (m.MSBigInteger, "CAST(t.col AS SIGNED INTEGER)"),
-            (m.MSBigInteger(unsigned=False), "CAST(t.col AS SIGNED INTEGER)"),
-            (m.MSBigInteger(unsigned=True), "CAST(t.col AS UNSIGNED INTEGER)"),
-            # this is kind of sucky.  thank you default arguments!
-            (NUMERIC, "CAST(t.col AS DECIMAL)"),
-            (DECIMAL, "CAST(t.col AS DECIMAL)"),
-            (Numeric, "CAST(t.col AS DECIMAL)"),
-            (m.MSNumeric, "CAST(t.col AS DECIMAL)"),
-            (m.MSDecimal, "CAST(t.col AS DECIMAL)"),
-            (TIMESTAMP, "CAST(t.col AS DATETIME)"),
-            (DATETIME, "CAST(t.col AS DATETIME)"),
-            (DATE, "CAST(t.col AS DATE)"),
-            (TIME, "CAST(t.col AS TIME)"),
-            (DateTime, "CAST(t.col AS DATETIME)"),
-            (Date, "CAST(t.col AS DATE)"),
-            (Time, "CAST(t.col AS TIME)"),
-            (DateTime, "CAST(t.col AS DATETIME)"),
-            (Date, "CAST(t.col AS DATE)"),
-            (m.MSTime, "CAST(t.col AS TIME)"),
-            (m.MSTimeStamp, "CAST(t.col AS DATETIME)"),
-            (String, "CAST(t.col AS CHAR)"),
-            (Unicode, "CAST(t.col AS CHAR)"),
-            (UnicodeText, "CAST(t.col AS CHAR)"),
-            (VARCHAR, "CAST(t.col AS CHAR)"),
-            (NCHAR, "CAST(t.col AS CHAR)"),
-            (CHAR, "CAST(t.col AS CHAR)"),
-            (m.CHAR(charset="utf8"), "CAST(t.col AS CHAR CHARACTER SET utf8)"),
-            (CLOB, "CAST(t.col AS CHAR)"),
-            (TEXT, "CAST(t.col AS CHAR)"),
-            (m.TEXT(charset="utf8"), "CAST(t.col AS CHAR CHARACTER SET utf8)"),
-            (String(32), "CAST(t.col AS CHAR(32))"),
-            (Unicode(32), "CAST(t.col AS CHAR(32))"),
-            (CHAR(32), "CAST(t.col AS CHAR(32))"),
-            (m.MSString, "CAST(t.col AS CHAR)"),
-            (m.MSText, "CAST(t.col AS CHAR)"),
-            (m.MSTinyText, "CAST(t.col AS CHAR)"),
-            (m.MSMediumText, "CAST(t.col AS CHAR)"),
-            (m.MSLongText, "CAST(t.col AS CHAR)"),
-            (m.MSNChar, "CAST(t.col AS CHAR)"),
-            (m.MSNVarChar, "CAST(t.col AS CHAR)"),
-            (LargeBinary, "CAST(t.col AS BINARY)"),
-            (BLOB, "CAST(t.col AS BINARY)"),
-            (m.MSBlob, "CAST(t.col AS BINARY)"),
-            (m.MSBlob(32), "CAST(t.col AS BINARY)"),
-            (m.MSTinyBlob, "CAST(t.col AS BINARY)"),
-            (m.MSMediumBlob, "CAST(t.col AS BINARY)"),
-            (m.MSLongBlob, "CAST(t.col AS BINARY)"),
-            (m.MSBinary, "CAST(t.col AS BINARY)"),
-            (m.MSBinary(32), "CAST(t.col AS BINARY)"),
-            (m.MSVarBinary, "CAST(t.col AS BINARY)"),
-            (m.MSVarBinary(32), "CAST(t.col AS BINARY)"),
-            (Interval, "CAST(t.col AS DATETIME)"),
-        ]
-
-        for type_, expected in specs:
-            self.assert_compile(cast(t.c.col, type_), expected)
+        self.assert_compile(cast(t.c.col, type_), expected)
 
     def test_cast_type_decorator(self):
         class MyInteger(sqltypes.TypeDecorator):
@@ -618,33 +640,29 @@ class SQLTest(fixtures.TestBase, AssertsCompiledSQL):
                 "(foo + 5)",
             )
 
-    def test_unsupported_casts(self):
+    m = mysql
+
+    @testing.combinations(
+        (m.MSBit, "t.col"),
+        (FLOAT, "t.col"),
+        (Float, "t.col"),
+        (m.MSFloat, "t.col"),
+        (m.MSDouble, "t.col"),
+        (m.MSReal, "t.col"),
+        (m.MSYear, "t.col"),
+        (m.MSYear(2), "t.col"),
+        (Boolean, "t.col"),
+        (BOOLEAN, "t.col"),
+        (m.MSEnum, "t.col"),
+        (m.MSEnum("1", "2"), "t.col"),
+        (m.MSSet, "t.col"),
+        (m.MSSet("1", "2"), "t.col"),
+    )
+    def test_unsupported_casts(self, type_, expected):
 
         t = sql.table("t", sql.column("col"))
-        m = mysql
-
-        specs = [
-            (m.MSBit, "t.col"),
-            (FLOAT, "t.col"),
-            (Float, "t.col"),
-            (m.MSFloat, "t.col"),
-            (m.MSDouble, "t.col"),
-            (m.MSReal, "t.col"),
-            (m.MSYear, "t.col"),
-            (m.MSYear(2), "t.col"),
-            (Boolean, "t.col"),
-            (BOOLEAN, "t.col"),
-            (m.MSEnum, "t.col"),
-            (m.MSEnum("1", "2"), "t.col"),
-            (m.MSSet, "t.col"),
-            (m.MSSet("1", "2"), "t.col"),
-        ]
-
-        for type_, expected in specs:
-            with expect_warnings(
-                "Datatype .* does not support CAST on MySQL;"
-            ):
-                self.assert_compile(cast(t.c.col, type_), expected)
+        with expect_warnings("Datatype .* does not support CAST on MySQL;"):
+            self.assert_compile(cast(t.c.col, type_), expected)
 
     def test_no_cast_pre_4(self):
         self.assert_compile(
@@ -910,3 +928,28 @@ class InsertOnDuplicateTest(fixtures.TestBase, AssertsCompiledSQL):
             "ON DUPLICATE KEY UPDATE bar = %s"
         )
         self.assert_compile(stmt, expected_sql)
+
+    def test_update_sql_expr(self):
+        stmt = insert(self.table).values(
+            [{"id": 1, "bar": "ab"}, {"id": 2, "bar": "b"}]
+        )
+        stmt = stmt.on_duplicate_key_update(
+            bar=func.coalesce(stmt.inserted.bar),
+            baz=stmt.inserted.baz + "some literal",
+        )
+        expected_sql = (
+            "INSERT INTO foos (id, bar) VALUES (%s, %s), (%s, %s) ON "
+            "DUPLICATE KEY UPDATE bar = coalesce(VALUES(bar)), "
+            "baz = (concat(VALUES(baz), %s))"
+        )
+        self.assert_compile(
+            stmt,
+            expected_sql,
+            checkparams={
+                "id_m0": 1,
+                "bar_m0": "ab",
+                "id_m1": 2,
+                "bar_m1": "b",
+                "baz_1": "some literal",
+            },
+        )
